# Generated by Django 3.2.13 on 2022-05-02 20:16

from django.db import migrations, models
import django.db.models.deletion
import pretalx_hitalx.models


class Migration(migrations.Migration):

    dependencies = [
        ('event', '0029_event_domain'),
        ('person', '0025_drop_unused_speakerinformation_fields'),
        ('pretalx_hitalx', '0002_auto_20220427_0849'),
    ]

    operations = [
        migrations.CreateModel(
            name='Tour',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False)),
                ('description', models.TextField()),
                ('departure_time', models.DateTimeField()),
                ('start_location', models.TextField()),
                ('type', models.TextField(choices=[(pretalx_hitalx.models.TourType['SHUTTLE'], 'Shuttle'), (pretalx_hitalx.models.TourType['BASSLINER'], 'Bassliner')])),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='event.event')),
                ('passengers', models.ManyToManyField(related_name='tours', to='person.SpeakerProfile')),
            ],
        ),
    ]



from enum import Enum

from django.db import models
from django.utils.translation import ugettext as _
from pretalx.common.mixins.models import LogMixin
from pretalx.event.models import Event
from pretalx.person.models import User, SpeakerProfile


class TourType(Enum):
    BASSLINER = 'BASSLINER'
    SHUTTLE = 'SHUTTLE'


TOUR_TYPE_CHOICES = [
    (TourType.SHUTTLE.value, _('Shuttle')),
    (TourType.BASSLINER.value, _('Bassliner')),
]


class ExpenseItem(LogMixin, models.Model):
    description = models.TextField(blank=False, null=False)
    amount = models.DecimalField(
        blank=False, null=False, decimal_places=2, max_digits=60
    )
    speaker = models.ForeignKey(User, on_delete=models.CASCADE, related_name="expenses")
    notes = models.TextField(blank=True, default="")
    reference = models.URLField(blank=True, null=True, verbose_name=_("Reference URL"))
    paid = models.BooleanField(default=False)


class Tour(models.Model):
    description = models.TextField(blank=False, null=False)
    departure_time = models.DateTimeField(null=False, blank=False)
    start_location = models.TextField(null=False, blank=False)
    passengers = models.ManyToManyField(SpeakerProfile, related_name='tours')
    type = models.TextField(choices=TOUR_TYPE_CHOICES)
    event = models.ForeignKey(Event, on_delete=models.CASCADE)

    def __str__(self):
        return f'{self.type} - {self.description} - {self.start_location} - {self.departure_time}'



from django import forms
from django.forms import CharField, ModelForm
from django_scopes.forms import SafeModelMultipleChoiceField
from pretalx.common.mixins.forms import ReadOnlyFlag
from pretalx.person.models import SpeakerProfile

from .models import ExpenseItem
from .models import Tour


class SpeakerExpenseForm(ReadOnlyFlag, ModelForm):
    description = CharField()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["speaker"].disabled = True
        self.fields["paid"].disabled = True

    class Meta:
        model = ExpenseItem
        fields = ["speaker", "description", "amount", "paid", "reference", "notes"]
        readonly_fields = ["speaker", "paid"]


class SpeakerToursForm(ReadOnlyFlag, ModelForm):
    tours = forms.ModelMultipleChoiceField(
         queryset=Tour.objects.all(),
         widget=forms.SelectMultiple(attrs={"class": "select2"}),
    )

    def save(self, *args, **kwargs):
        instance = self.instance
        if "tours" in self.changed_data:
            instance.tours.set(self.cleaned_data["tours"])

    def __init__(self, *args, **kwargs):
        super(SpeakerToursForm, self).__init__(*args, **kwargs)
        self.initial['tours'] = kwargs['instance'].tours.all()

    class Meta:
        model = SpeakerProfile
        fields = ["tours"]
        field_classes = {
            "tours": SafeModelMultipleChoiceField,
        }


class TourForm(ReadOnlyFlag, ModelForm):
    description = CharField()
    start_location = CharField()
    departure_time = forms.DateTimeField()

    def __init__(self, *args, user=None, locales=None, organiser=None, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["departure_time"].widget.attrs["class"] = "datetimepickerfield"

    class Meta:
        model = Tour
        fields = ["description", "departure_time", "start_location", "type", "event"]
        widgets = {'event': forms.HiddenInput()}



from decimal import Decimal

from django import template
from django.db.models import Sum

register = template.Library()


@register.filter(name="total_expenses")
def total_expenses(user):
    amount = Decimal(user.expenses.all().aggregate(Sum("amount"))["amount__sum"] or 0)
    return amount


@register.filter(name="paid_expenses")
def paid_expenses(user):
    amount = Decimal(
        user.expenses.filter(paid=True).aggregate(Sum("amount"))["amount__sum"] or 0
    )
    return amount


@register.filter(name="euro_amount")
def euro_amount(amount):
    return f"{amount:.2f} €".replace(".", ",")
